@@ -212,11 +212,6 @@ class RpmHelper(object):
     if rpmutils_present:
         _rpm_util = rpmupdates.Updates([], [])
 
-    @classmethod
-    def prepcache(cls):
-        """Prepare the yum cache."""
-        CommandRunner("yum -y makecache").run()
-
     @classmethod
     def compare_rpm_versions(cls, v1, v2):
         """Compare two RPM version strings.
@@ -290,7 +285,7 @@ class RpmHelper(object):
                    e.g., httpd-2.2.22
                    e.g., httpd-2.2.22-1.fc16
         """
-        cmd_str = "yum -C -y --showduplicates list available %s" % pkg
+        cmd_str = "yum -y --showduplicates list available %s" % pkg
         command = CommandRunner(cmd_str).run()
         return command.status == 0
 
@@ -417,8 +412,6 @@ class PackagesHandler(object):
         # collect pkgs for batch processing at end
         installs = []
         downgrades = []
-        # update yum cache
-        RpmHelper.prepcache()
         for pkg_name, versions in packages.iteritems():
             ver = RpmHelper.newest_rpm_version(versions)
             pkg = "%s-%s" % (pkg_name, ver) if ver else pkg_name
@@ -461,7 +454,8 @@ class PackagesHandler(object):
         # TODO(asalkeld) support versions
         pkg_list = ' '.join([p for p in packages])
 
-        cmd_str = 'apt-get -y install %s' % pkg_list
+        cmd_str = 'DEBIAN_FRONTEND=noninteractive apt-get -y install %s' % \
+                  pkg_list
         CommandRunner(cmd_str).run()
 
     # map of function pointers to handle different package managers
@@ -562,23 +556,31 @@ class SourcesHandler(object):
     def _url_to_tmp_filename(self, url):
         tempdir = tempfile.mkdtemp()
         atexit.register(lambda: shutil.rmtree(tempdir, True))
-        sp = url.split('/')
-        if 'https://github.com' in url:
-            if 'zipball' == sp[-2]:
-                return '%s/%s-%s.zip' % (tempdir, sp[-3], sp[-1])
-            elif 'tarball' == sp[-2]:
-                return '%s/%s-%s.tar.gz' % (tempdir, sp[-3], sp[-1])
-            else:
-                pass
-
-        return '%s/%s' % (tempdir, sp[-1])
+        name = os.path.basename(url)
+        return os.path.join(tempdir, name)
 
     def _splitext(self, path):
         (r, ext) = os.path.splitext(path)
         return (r, ext.lower())
 
-    def _source_type(self, name):
-        (r, ext) = self._splitext(name)
+    def _github_ball_type(self, url):
+        ext = ""
+        if url.endswith('/'):
+            url = url[0:-1]
+        sp = url.split('/')
+        if len(sp) > 2:
+            http = sp[0].startswith('http')
+            github = sp[2].endswith('github.com')
+            btype = sp[-2]
+            if http and github:
+                if 'zipball' == btype:
+                    ext = '.zip'
+                elif 'tarball' == btype:
+                    ext = '.tgz'
+        return ext
+
+    def _source_type(self, url):
+        (r, ext) = self._splitext(url)
         if ext == '.gz':
             (r, ext2) = self._splitext(r)
             if ext2 == '.tar':
@@ -587,12 +589,15 @@ class SourcesHandler(object):
             (r, ext2) = self._splitext(r)
             if ext2 == '.tar':
                 ext = '.tbz2'
+        elif ext == "":
+            ext = self._github_ball_type(url)
+
         return ext
 
     def _apply_source_cmd(self, dest, url):
         cmd = ""
         basename = os.path.basename(url)
-        stype = self._source_type(basename)
+        stype = self._source_type(url)
         if stype == '.tgz':
             cmd = "wget -q -O - '%s' | gunzip | tar -xvf -" % url
         elif stype == '.tbz2':
@@ -608,8 +613,6 @@ class SourcesHandler(object):
         elif stype == '.bz2':
             (r, ext) = self._splitext(basename)
             cmd = "wget -q -O - '%s' | bunzip2 > '%s'" % (url, r)
-        else:
-            pass
 
         if cmd != '':
             cmd = "mkdir -p '%s'; cd '%s'; %s" % (dest, dest, cmd)
@@ -843,15 +846,6 @@ class CommandsHandler(object):
         cwd = None
         env = properties.get("env", None)
 
-        if "test" in properties:
-            test_status = CommandRunner(properties["test"]).run().status
-            if test_status != 0:
-                LOG.info("%s test returns false, skipping command"
-                         % command_label)
-                return
-            else:
-                LOG.debug("%s test returns true, proceeding" % command_label)
-
         if "cwd" in properties:
             cwd = os.path.expanduser(properties["cwd"])
             if not os.path.exists(cwd):
@@ -859,11 +853,23 @@ class CommandsHandler(object):
                           "%s path does not exist" % cwd)
                 return
 
+        if "test" in properties:
+            test = CommandRunner(properties["test"])
+            test_status = test.run('root', cwd, env).status
+            if test_status != 0:
+                LOG.info("%s test returns false, skipping command"
+                         % command_label)
+                return
+            else:
+                LOG.debug("%s test returns true, proceeding" % command_label)
+
         if "command" in properties:
             try:
-                # TODO(pfreund) aws doc : "Either an array or a string
-                # specifying the command to run" Need the array.
-                command = CommandRunner(properties["command"])
+                command = properties["command"]
+                if isinstance(command, list):
+                    escape = lambda x: '"%s"' % x.replace('"', '\\"')
+                    command = ' '.join(map(escape, command))
+                command = CommandRunner(command)
                 command.run('root', cwd, env)
                 command_status = command.status
             except OSError as e:
@@ -1049,6 +1055,36 @@ class Metadata(object):
             'DescribeStackResourceResult']['StackResourceDetail']
         return resource_detail['Metadata']
 
+    def get_nova_meta(self,
+                      cache_path='/var/lib/heat-cfntools/nova_meta.json'):
+        """Get nova's meta_data.json and cache it.
+
+        Since this is called repeatedly return the cached metadata,
+        if we have it.
+        """
+
+        url = 'http://169.254.169.254/openstack/2012-08-10/meta_data.json'
+        if not os.path.exists(cache_path):
+            CommandRunner('wget -O %s %s' % (cache_path, url)).run()
+        try:
+            with open(cache_path) as fd:
+                try:
+                    return json.load(fd)
+                except ValueError:
+                    pass
+        except IOError:
+            pass
+        return None
+
+    def get_tags(self):
+        """Get the tags for this server."""
+        tags = {}
+        md = self.get_nova_meta()
+        if md is not None:
+            tags.update(md.get('meta', {}))
+            tags['InstanceId'] = md['uuid']
+        return tags
+
     def retrieve(
             self,
             meta_str=None,
@@ -1139,9 +1175,47 @@ class Metadata(object):
     def __str__(self):
         return json.dumps(self._metadata)
 
-    def display(self):
-        if self._metadata is not None:
-            print str(self)
+    def display(self, key=None):
+        """Print the metadata to the standard output stream. By default the
+        full metadata is displayed but the ouptut can be limited to a specific
+        with the <key> argument.
+
+        Arguments:
+            key -- the metadata's key to display, nested keys can be specified
+                   separating them by the dot character.
+                        e.g., "foo.bar"
+                   If the key contains a dot, it should be surrounded by single
+                   quotes
+                        e.g., "foo.'bar.1'"
+        """
+        if self._metadata is None:
+            return
+
+        if key is None:
+            print(str(self))
+            return
+
+        value = None
+        md = self._metadata
+        while True:
+            key_match = re.match(r'^(?:(?:\'([^\']+)\')|([^\.]+))(?:\.|$)',
+                                 key)
+            if not key_match:
+                break
+
+            k = key_match.group(1) or key_match.group(2)
+            if isinstance(md, dict) and k in md:
+                key = key.replace(key_match.group(), '')
+                value = md = md[k]
+            else:
+                break
+
+        if key != '':
+            value = None
+
+        if value is not None:
+            print(json.dumps(value))
+
         return
 
     def _is_valid_metadata(self):
